@@ -61,6 +61,9 @@
 #ifdef HAVE_SYS_ACL_H
 #include <sys/acl.h>
 #endif /* HAVE_SYS_ACL_H */
+#if HAVE_FTS_H
+#include <fts.h>
+#endif /* HAVE_FTS_H */
 
 #if !HAVE_DECL_SETENV
 extern int setenv (const char *name, const char *value, int replace);
@@ -662,9 +665,6 @@ int WRAP_FSTATAT64 FSTATAT64_ARG(int ver,
 
 #endif /* STAT64_SUPPORT */
 
-
-
-
 /*************************************************************/
 /*
   Wrapped functions general info:
@@ -856,6 +856,10 @@ int chmod(const char *path, mode_t mode){
   r=next_chmod(path, mode);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 
@@ -884,6 +888,10 @@ int fchmod(int fd, mode_t mode){
   r=next_fchmod(fd, mode);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 
@@ -916,6 +924,10 @@ int fchmodat(int dir_fd, const char *path, mode_t mode, int flags) {
   r=next_fchmodat(dir_fd, path, mode, flags);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 #endif /* HAVE_FCHMODAT */
@@ -1477,3 +1489,20 @@ int acl_set_file(const char *path_p, acl_type_t type, acl_t acl) {
   return -1;
 }
 #endif /* HAVE_SYS_ACL_H */
+
+#ifdef HAVE_FTS_READ
+FTSENT *fts_read(FTS *ftsp) {
+  FTSENT *r;
+
+  r=next_fts_read(ftsp);
+  if(r && r->fts_statp) {  /* Should we bother checking fts_info here? */
+# ifndef STUPID_ALPHA_HACK
+    send_get_stat64(r->fts_statp);
+# else
+    send_get_stat64(r->fts_statp, _STAT_VER);
+# endif
+  }
+
+  return r;
+}
+#endif /* HAVE_FTS_READ */
